/* ----------------------------------------------------------------------
 * cachectl.S
 *
 * Copyright (C) 1994 by Waldorf Electronics,
 * written by Ralf Baechle and Andreas Busse
 *
 * This file is subject to the terms and conditions of the
 * GNU General Public License. See the file COPYING in the
 * main directory of this archive for more details.
 * ---------------------------------------------------------------------- */

#include "asm/regdef.h"

#if 0
#include <cachectl.h>

 /*
  * Caches to flush
  */
 #define ICACHE          0x1             /* flush instruction cache */
 #define DCACHE          0x2             /* flush data cache        */
 #define BCACHE          (ICACHE|DCACHE) /* flush both caches       */

 /*
  * Caching modes
  */
 #define CACHEABLE       0               /* mark cacheable          */
 #define UNCACHEABLE     1               /* mark noncacheable       */
 #define CACHELINES	512             /* number of cachelines    */

 /*
  * Flush instruction/data caches
  *
  * Parameters: a0 - starting address to flush
  *             a1 - size of area to be flushed
  *             a2 - which caches to be flushed
  *
  * FIXME:      - ignores parameters
  *             - doesn't know about second level caches
  */

			 .set	noreorder
			 .globl	_cacheflush
			 .text
 _cacheflush:
			 /*
			  * Flush the instruction cache
			  */
	
			 lui	t0,0x8000
			 li	t1,CACHELINES-1
1: 			 cache	0,0(t0)
			 cache	0,32(t0)
			 cache	0,64(t0)
			 cache	0,96(t0)
			 cache	0,128(t0)
			 cache	0,160(t0)
			 cache	0,192(t0)
			 cache	0,224(t0)
			 cache	0,256(t0)
			 cache	0,288(t0)
			 cache	0,320(t0)
			 cache	0,352(t0)
			 cache	0,384(t0)
			 cache	0,416(t0)
			 cache	0,448(t0)
			 cache	0,480(t0)
			 addiu	t0,t0,512
			 bne	zero,t1,1b
			 subu	t1,t1,1

			 /*
			  * Flush the data cache
			  */
			 lui	t0,0x8000
			 li	t1,CACHELINES-1
1:                       cache   1,0(t0)
			 cache   1,32(t0)
			 cache   1,64(t0)
			 cache   1,96(t0)
			 cache   1,128(t0)
			 cache   1,160(t0)
			 cache   1,192(t0)
			 cache   1,224(t0)
			 cache   1,256(t0)
			 cache   1,288(t0)
			 cache   1,320(t0)
			 cache   1,352(t0)
			 cache   1,384(t0)
			 cache   1,416(t0)
			 cache   1,448(t0)
			 cache   1,480(t0)
			 addiu   t0,t0,512
			 bne     zero,t1,1b
			 subu    t1,t1,1

			 j	ra
			 nop

#else

/*
 * This is the newer function. It should handle the differences
 * between the various Mips processors...
 */
	
/*
 * Cache Operations
 */
	
#define INDEX_INVALIDATE_I      0x00
#define INDEX_WB_INVALIDATE_D   0x01
#define INDEX_INVALIDATE_SI     0x02	
#define INDEX_WB_INVALIDATE_SD  0x03
#define INDEX_LOAD_TAG_I        0x04
#define INDEX_LOAD_TAG_D        0x05
#define INDEX_LOAD_TAG_SI       0x06
#define INDEX_LOAD_TAG_SD       0x07	
#define INDEX_STORE_TAG_I       0x08
#define INDEX_STORE_TAG_D       0x09
#define INDEX_STORE_TAG_SI      0x0a
#define INDEX_STORE_TAG_SD      0x0b
#define CREATE_DIRTY_EXCL_D     0x0d	
#define CREATE_DIRTY_EXCL_SD    0x0f
#define HIT_INVALIDATE_I        0x10
#define HIT_INVALIDATE_D	0x11
#define HIT_INVALIDATE_SI       0x12
#define HIT_INVALIDATE_SD	0x13
#define FILL_I                  0x14
#define HIT_WB_INVALIDATE_D	0x15
#define HIT_WB_INVALIDATE_SD	0x17
#define HIT_WRITEBACK_I         0x18
#define HIT_WRITEBACK_D         0x19
#define HIT_WRITEBACK_SD        0x1b
#define HIT_SETVIRTUAL_SI       0x1e
#define HIT_SETVIRTUAL_SD       0x1f	

/* Memory segments */

#define KSEG0                   0x80000000
#define KSEG1                   0xa0000000

#define L2CACHE_SIZE		0x00200000
#define L2FLUSH_BASE		0x80200000
#define L2FLUSH_END		L2FLUSH_BASE + L2CACHE_SIZE

 /*
  * Flush complete instruction/data caches
  *
  * Parameters: a0 - starting address to flush
  *	        a1 - size of area to be flushed
  *             a2 - which caches to be flushed
  *
  * NOTE: If we speak about s-cache here, we mean
  *       *real* scache, ie. directly connected
  *       to the processor. L2 cache controlled
  *       by any support chipsets aren't flushed
  *       here!
  *
  * FIXME: parameters ignored
  *
  */

	.globl	_cacheflush
	.text
	
_cacheflush:
	lw	t6, _scache_size	# get s-cache info
	bnez	t6, 300f	        # we have scache, skip I and D
					# I mean: REAL scache

	/* get d-cache info */
	
100:	lw	t4, _dcache_size
	lw	t5, _dcache_linesize
	beqz	t4, 200f                # no dcache, skip

	/* flush d-cache */
	
	li	t0, KSEG0		# start at the bottom of KSEG0
	addu	t1, t0, t4
	subu	t1, t5

	.set	noreorder
101:	cache	INDEX_WB_INVALIDATE_D,0(t0)	# Invalidate d-cache line.
	bne	t0, t1, 101b
	addu	t0, t5
	.set	reorder

	/* get i-cache info */
	
200:	lw	t2, _icache_size	# get i-cache info
	lw	t3, _icache_linesize
	beqz	t2, 400f                # no icache, skip

	/* flush i-cache */
	
	li	t0, KSEG0		# start at KSEG0
	addu	t1, t0, t2
	subu	t1, t3

	.set	noreorder
201:	cache	INDEX_INVALIDATE_I,0(t0)	# Flush out the icache line
	bne	t0, t1, 201b
	addu	t0, t3
	.set	reorder
	b	400f                    # ready

	/* flush s-cache (I mean *real* scache !!) */

300:	lw	t7, _scache_linesize
	li	t0, KSEG0		# start at the bottom of kseg0
	addu	t1, t0, t6
	subu	t1, t7
	.set	noreorder
301:	cache	INDEX_WB_INVALIDATE_SD,0(t0)	# Flush out the SD line
	bne	t0, t1, 301b
	addu	t0, t7
	.set	reorder


	/*
	 * flush secondary cache controlled by some chip sets.
 	 * Since we don't know how large it is we simply assume
	 * the usual maximum size of 2 Megs. I've never heard
	 * about larger L2 caches of this kind...
	 */
#if 0
400:	j	ra                      # ready
#else
400:	li	t0,L2FLUSH_BASE		# start at the bottom
	li	t1,L2FLUSH_END-4

	.set noreorder
401:	lw	t2,0(t0)		# read the complete space
	bne	t0,t1,401b			
	addiu	t0,t0,4			# bd slot	
	.set reorder

	/* that's it. */

	j 	ra

#endif
#endif	
