/* ----------------------------------------------------------------------
 * dumptlb.c
 *
 * Copyright (C) 1994 by Waldorf Electronics,
 * written by Ralf Baechle and Andreas Busse
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 * ---------------------------------------------------------------------- */

#include <asm/mipsregs.h>
#include <linux/string.h>

#include "console.h"
#include <arc/arc.h>

static char temp[1024];

#define VALID 0x02		/* valid bit */

void
dump_tlb(void)
{
  int	i;
  int	wired;
  int	dummy;
 
  unsigned long entry_hi[2];
  unsigned long entry_lo0[2];
  unsigned long entry_lo1[2];

  printf("\r\n\r\nTLB DUMP:\r\n");
  wired = read_32bit_cp0_register(CP0_WIRED);
  printf("Wired: %d\r\n\r\n", wired);
  
  for(i=0 ; i<48; i++)		/* print all TLB entries */
    {
      write_32bit_cp0_register(CP0_INDEX, i);
      __asm__ __volatile__("tlbr");
      
      read_64bit_cp0_register(entry_hi,CP0_ENTRYHI);
      read_64bit_cp0_register(entry_lo0,CP0_ENTRYLO0);
      read_64bit_cp0_register(entry_lo1,CP0_ENTRYLO1);
      
      if ((entry_lo0[1] && VALID) || (entry_lo1[1] && VALID))
	{
	  printf("TLB #%d:\r\n",i);
	  printf("\tEntryHi : %08x:%08x\t",entry_hi[1],entry_hi[0]);
	  printf("(R=%1x VADR=%02x:%08x ASID=%04x)\r\n",
		 entry_hi[1] >> 30,
		 entry_hi[1] & 0xff,
		 entry_hi[0] & 0xffffe000,
		 entry_hi[0] & 0xff);
	  printf("\tEntryLo0: %08x:%08x\t",entry_lo0[1],entry_lo0[0]);
	  printf("(PADR=%02x:%08x C=%d D=%d V=%d G=%d)\r\n",
		 (entry_lo0[0] & 0x3c000000) >> 25,
		 (entry_lo0[0] & 0x03ffffc0) << 6,
		 (entry_lo0[0] & 0x00000038) >> 3,
		 (entry_lo0[0] & 0x00000004) >> 2,
		 (entry_lo0[0] & 0x00000002) >> 1,
		 (entry_lo0[0] & 0x00000001));
	  printf("\tEntryLo1: %08x:%08x\t",entry_lo1[1],entry_lo1[0]);
	  printf("(PADR=%02x:%08x C=%d D=%d V=%d G=%d)\r\n",
		 (entry_lo1[0] & 0x3c000000) >> 25,
		 (entry_lo1[0] & 0x03ffffc0) << 6,
		 (entry_lo1[0] & 0x00000038) >> 3,
		 (entry_lo1[0] & 0x00000004) >> 2,
		 (entry_lo1[0] & 0x00000002) >> 1,
		 (entry_lo1[0] & 0x00000001));
	  printf("\r\nPageMask: %08x\r\n\r",
		 read_32bit_cp0_register(CP0_PAGEMASK));
	  printf("Press a key for next\r\n");
	  getchar();
	}
    }
}
